// Actions must be part of the com.tivoli.zce.actions.libs package.
package com.tivoli.zce.actions.libs;

// Minimal set of State Correlation imports
import com.tivoli.zce.IRule;
import com.tivoli.zce.ParserException;
import com.tivoli.zce.CorrelatorException;
import com.tivoli.zce.engine.EventList;
import com.tivoli.zce.engine.Event;
import com.tivoli.zce.actions.DefaultActionHandler;
import com.tivoli.zce.persistence.PHandler;

import com.tivoli.zce.actions.TecEventUtil;



/******************************************************************************
* A sample action which creates a new event
*
* <p>This action generates a new TEC_Notice event. The host name of the original
* event is used as the host name of the new event. The message field of the new 
* event is given text indicating it was generated by the state correlation engine
* and the origin field is set to 'SCE'.</p>
*
* <p>Once the new event is created it must be forwarded on to any subsequent actions
* defined in the rule before being sent to TEC. This is accomplished by calling the 
* <code>TecEventUtil.forwardTecEvent()</code> utility method which does all the 
* additional preparation needed for creating a TEC event and forwarding it.
* Directly calling the <code>forward()</code> method with a newly created event 
* will fail. Existing events have already been prepared by the time they are 
* passed to an action.</p>
*
* <p>The new event may be persisted so it can be recovered in the event of a
* State Correlation failure. The new event should be persisted after all changes
* have been made (the persisted copy cannot be updated) and is persisted by
* calling <code>PHandler.addEvent()</code>. The state correlation engine's
* persistence mechanism must be enabled before this will have any effect.  
* Persistence is enabled by setting the BufferEvents=YES entry in the 
* configuration file and specifying the associated BufEvtPath entry.</p>
*
* <p>Additional processing can be done on the original event before forwarding it
* to the next action. If desired, the original event can be dropped by not making
* the forward call. If the event is dropped a call to 
* <code>PHandler.removeEvent_termAction()</code should be made to remove the event
* from the persistent store.</p>
*
* @see DefaultActionHandler
* @see com.tivoli.zce.actions.TecEventUtil
* @see ITecEventAttributes
* @see PHandler
*
******************************************************************************/

public class CreateNewEventSample extends DefaultActionHandler implements ITecEventAttributes 
{

   // A default host name to use if the event does not contain one
   private final String DEFAULT_HOSTNAME = "'no host'";

/******************************************************************************
* Processes a list of events
*
* <p>Takes the provided list of events and processes each event individually by
* passing it to this actions processEvent() method. </p>
*
* <p> The event list represents the collection of events needed to trigger a
* rule by the State Correlation Engine.  This method is typically called when 
* a state based rule is used and the event list contains the events from the 
* event which triggered the state machine for the rule to the last events 
* processed by the state machine.
*
* @param eventList the list of events to process
*
* @see #processEvent
* @see DefaultActionHandler
* @see EventList
*
******************************************************************************/
   public void processEvents(EventList eventList) throws Exception 
   {
      if (eventList != null)
      {
         try
         {
            for (int i = 0; i < eventList.size(); i++)
            {
               processEvent( eventList.eventAt( i));
            }
         }
         catch (Exception x)
         {
            x.printStackTrace();
         }
      }
   }


/******************************************************************************
* Processes a single event
*
* <p> Creates a new event of type <b>TEC_Notice</b> using the host 
* name of the original event if one is present.  The new event is given
* a message of "Generated by State Correlation" and an origin of "SCE".
* Both the new and the original events are forwarded to the next action
* in this rule for further processing.</p>
*
* @param event the event to process
*
* @see #processEvents
* @see DefaultActionHandler
* @see Event
*
******************************************************************************/
   public void processEvent(Event event) throws Exception 
   {
      Event  evt      = new Event();
      String hostname = DEFAULT_HOSTNAME;

      if (event!= null)
      {
         if (event.hasAttribute( HOSTNAME))
         {
            hostname = event.getString( HOSTNAME);
         }
      }

      evt.type = "TEC_Notice";
      evt.putItem( HOSTNAME, hostname);
      evt.putItem( MSG, "'Generated by State Correlation'");
      evt.putItem( ORIGIN, "SCE");

      // Send the new event and Continue processing the original event
      TecEventUtil.forwardTecEvent( this, evt, event);

      // If desired the new event may be persisted so it may be 
      // recovered in the event of a state correlation failure
      PHandler.addEvent( evt);

      // Continue processing the original event
      // Remove the forward() call if the original event is to be dropped
      forward(event);

      // If not forwarding then the following line should be 
      // uncommented so that the event is removed from the persistent store

      // PHandler.removeEvent_termAction(event);
   }



/******************************************************************************
* Parses any parameters which are part of the action definition
*
* <p> All parameters defined in the rule XML for this action are passed
* in the string paramSz.  For this particular action, no parameters are
* expected so no parsing is done.</p>
*
* <p> This method is called once for each rule containing this action.  If 
* the action is shared (<code> singleInstance="true"</code>) then this 
* method is only called once.</p>
*
* @param rule the specific instance of the rule containing this action
* @param paramSz the parameters defined for this action in this rule instance
*
* @see DefaultActionHandler
* @see IRule
*
******************************************************************************/
   public Object doParse(IRule rule, String paramSz) throws ParserException 
   {
      return null;
   }
}



