// Actions must be part of the com.tivoli.zce.actions.libs package.
package com.tivoli.zce.actions.libs;

// Minimal set of State Correlation imports
import com.tivoli.zce.IRule;
import com.tivoli.zce.ParserException;
import com.tivoli.zce.CorrelatorException;
import com.tivoli.zce.engine.EventList;
import com.tivoli.zce.engine.Event;
import com.tivoli.zce.actions.DefaultActionHandler;


/******************************************************************************
* A sample action which escalates the severity of an incoming event
*
* <p>This action modifies the value of the event attribute "severity" by
* setting the severity to the next higher entry in the default TEC severity list.
*
* <ul>
* <li> UNKNOWN  --> WARNING
* <li> WARNING  --> MINOR
* <li> MINOR    --> CRITICAL
* <li> CRITICAL --> FATAL
* </ul>
* </p>
*
* <p> Events with a severity of "FATAL" are not changed. Events with a severity
* which is not one of the default TEC severities are also not changed.</p>
* <p> Events with no severity are given a severity of "UNKNOWN".</p>
*
* <p>Once the event has been updated it must be forwarded on to any subsequent 
* actions defined in the rule before being sent to TEC.  The event snapshot 
* must also be updated prior to exiting the last action in the rule. This can 
* be accomplished by either using the SendTecEvent action as the last action in
* the rule or by calling the <code>TecEventUtil.forwardTecEvent()</code> 
* utility method which does all the additional preparation needed for creating
* a TEC event and forwards the event.</p>
*
* @see DefaultActionHandler
* @see com.tivoli.zce.actions.TecEventUtil
* @see ITecEventAttributes
*
******************************************************************************/
public class EventModificationSample extends DefaultActionHandler implements ITecEventAttributes
{

/******************************************************************************
* Processes a list of events
*
* <p>Takes the provided list of events and processes each event individually by
* passing it to this actions processEvent() method. </p>
*
* <p> The event list represents the collection of events needed to trigger a
* rule by the State Correlation Engine.  This method is typically called when 
* a state based rule is used and the event list contains the events from the 
* event which triggered the state machine for the rule to the last events 
* processed by the state machine.
*
* @param eventList the list of events to process
*
* @see #processEvent
* @see DefaultActionHandler
* @see EventList
*
******************************************************************************/
    public void processEvents(EventList eventList) throws Exception 
    {
       if (eventList != null)
       {
          try
          {
            for (int i = 0; i < eventList.size(); i++)
            {
                processEvent( eventList.eventAt( i));
            }
          }
          catch (Exception x)
          {
             x.printStackTrace();
          }
       }
    }
    
    
/******************************************************************************
* Processes a single event
*
* <p>This action modifies the value of the event attribute "severity" by
* setting the severity to the next higher entry in the default TEC severity list.
*
* <ul>
* <li> UNKNOWN  --> WARNING
* <li> WARNING  --> MINOR
* <li> MINOR    --> CRITICAL
* <li> CRITICAL --> FATAL
* </ul>
* </p>
*
* <p> Events with a severity of "FATAL" are not changed. Events with a severity
* which is not one of the default TEC severities are also not changed.</p>
* <p> Events with no severity are given a severity of "UNKNOWN".</p>
*
* <p>The event is then forwarded to any subsequent actions.</p>
*
* <p>Since the event snapshot is not updated in this action it is assumed that
* the update will be done in a subsequent action.  If it is not updated in a 
* later action the change to the severity will be lost.</p>
*
* @param event the event to process
*
* @see #processEvents
* @see DefaultActionHandler
* @see Event
*
******************************************************************************/
    public void processEvent(Event event) throws Exception 
    {
	String severity = null;

	if (event!= null)
        {
           if (event.hasAttribute( SEVERITY))
           {
              severity = event.getString( SEVERITY);
              int i = 0;
              while (i < SEVERITY_LIST.length && !(severity.equals( SEVERITY_LIST[ i])))
              {
                 i++;
              }

              if (i < SEVERITY_LIST.length - 1)
              {
                 severity = SEVERITY_LIST[ i + 1];
              }

           }
           else
           {
              severity = SEVERITY_LIST[ 0];
           }
	}

        event.putItem( SEVERITY, severity);

        // Forward the event to any other actions
	forward(event);
    }


/******************************************************************************
* Parses any parameters which are part of the action definition
*
* <p> All parameters defined in the rule XML for this action are passed
* in the string paramSz.  For this particular action, no parameters are
* expected so no parsing is done.</p>
*
* <p> This method is called once for each rule containing this action.  If 
* the action is shared (<code> singleInstance="true"</code>) then this 
* method is only called once.</p>
*
* @param rule the specific instance of the rule containing this action
* @param paramSz the parameters defined for this action in this rule instance
*
* @see DefaultActionHandler
* @see IRule
*
******************************************************************************/
    public Object doParse(IRule rule, String paramSz) throws ParserException 
    {
	return null;
    }    
    
}
    


