// Actions must be part of the com.tivoli.zce.actions.libs package.
package com.tivoli.zce.actions.libs;

// Minimal set of State Correlation imports
import com.tivoli.zce.IRule;
import com.tivoli.zce.ParserException;
import com.tivoli.zce.CorrelatorException;
import com.tivoli.zce.engine.EventList;
import com.tivoli.zce.engine.Event;
import com.tivoli.zce.actions.DefaultActionHandler;

import java.util.Enumeration;
import java.util.NoSuchElementException;
import java.util.Properties;
import java.util.StringTokenizer;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.Statement;
import java.sql.ResultSet;
import java.sql.SQLException;



/******************************************************************************
* A sample action which updates an event based on the results of a JDBC lookup
*
* <p>This action uses the incoming event type and an attribute of the event to
* provide the key for a JDBC query.  The query returns the proper operator
* response for the event and appends it to the message field of the event.</p>
*
* <p>The <code>doParse()</code> method receives the information necessary for
* making the JDBC connection from parameters defined in the rule XML file.  
* These parameters include the URL of the database, the user name, the password
* and the JDBC driver to use.  Once this information is collected, a connection
* to the database is created along with a statement to use for the query.  These
* resources are closed during the <code>finalize()</code> method.</p>
*
* <p>Once the incoming event type is determined, a specific attribute for each
* event type is chosen to be part of the database look up key.  This allows 
* for different messages to be returned based on the event content.  For 
* example, a TEC_ITS_NODE_STATUS event with a nodestatus of DOWN can return a 
* different message than a TEC_ITS_NODE_STATUS event with a nodestatus of UP.
* Once the attribute value is determined a query is made using the event type
* and attribute value as the key.  The response value in the retrieved database
* row is used to augment the event message.</p>
*
* <p>This sample uses a database table EVNT_REF consisting of three columns: 
* EVENT_CLASS, STATUS, and RESPONSE.  All three columns are of type string 
* and the EVENT_CLASS and STATUS columns form the primary key for the table.
* The following rows were part of the table:</p>
*
*
* <pre>
* EVENT_CLASS                  STATUS                              RESPONSE
* -----------------------------------------------------------------------------------------------------------
* TEC_ITS_NODE_STATUS          DOWN                                Contact Fred in network support (xxx-xxxx)
* TEC_ITS_NODE_STATUS          UP                                  No action needed
* TEC_ITS_NODE_STATUS          MARGINAL                            Contact Fred in network support (xxx-xxxx)
* TEC_ITS_SUBNET_CONNECTIVITY  UNREACHABLE                         Contact Amy in network support (xxx-xxxx)
* TEC_ITS_SUBNET_CONNECTIVITY  REACHABLE_AGAIN                     No action necessary
* TEC_ITS_NODE_SERVICE_IMPACT  IBM_DB2                             Contact DB2 administrator
* TEC_ITS_NODE_SERVICE_IMPACT  IBM_WebSphere_MQ                    Contact MQ administrator
* TEC_ITS_NODE_SERVICE_IMPACT  IBM_WebSphere_Application_Server    Contact WebSphere administrator
* </pre>
* 
*
* @see DefaultActionHandler
* @see com.tivoli.zce.actions.TecEventUtil
* @see ITecEventAttributes
*
******************************************************************************/

public class JDBCAccessSample extends DefaultActionHandler implements ITecEventAttributes
{

   private Properties parameters_ = new Properties();

   private Connection dbCon_ = null;
   private Statement  stmt_  = null;

/******************************************************************************
* Processes a list of events
*
* <p>Takes the provided list of events and processes each event individually by
* passing it to this actions processEvent() method. </p>
*
* <p> The event list represents the collection of events needed to trigger a
* rule by the State Correlation Engine.  This method is typically called when 
* a state based rule is used and the event list contains the events from the 
* event which triggered the state machine for the rule to the last events 
* processed by the state machine.
*
* @param eventList the list of events to process
*
* @see #processEvent
* @see DefaultActionHandler
* @see EventList
*
******************************************************************************/

   public void processEvents(EventList eventList) throws Exception 
   {
      if (eventList != null)
      {
         try
         {
            for (int i = 0; i < eventList.size(); i++)
            {
               processEvent( eventList.eventAt( i));
            }
         }
         catch (Exception x)
         {
            x.printStackTrace();
         }
      }
   }

/******************************************************************************
* Processes a single event
*
* <p>This action modifies the value of the event attribute "msg" by
* looking up a canned response in a database using the event type and one other
* event attribute as the primary key.</p>
*
* <p>The event is then forwarded to any subsequent actions.</p>
*
* <p>Since the event snapshot is not updated in this action it is assumed that
* the update will be done in a subsequent action.  If it is not updated in a 
* later action the change to the message will be lost.</p>
*
* @param event the event to process
*
* @see #processEvents
* @see DefaultActionHandler
* @see Event
*
******************************************************************************/
   public void processEvent(Event event) throws Exception 
   {
      String    searchstr  = null;
      ResultSet results    = null;
      String    eventClass = null;
      String    eventAttr  = null;
      String    response   = null;
      String    msg        = null;

      if (event!= null && event.type != null)
      {

         eventClass = event.type;

         eventAttr = getPrimaryEventAttributeValue( event);

         if (eventAttr != null)
         {
            searchstr = "EVENT_CLASS = '" + event.type + "' AND STATUS = '" + eventAttr + "'";
         }
         else
         {
            searchstr = "EVENT_CLASS = '" + event.type + "'";
         }

         try
         {
            searchstr = "SELECT * FROM EVNT_REF WHERE " + searchstr;

            results = stmt_.executeQuery( searchstr);

            results.next();

            response = results.getString( "RESPONSE");
         }
         catch (SQLException ex)
         {
            ex.printStackTrace();
         }
         catch (Exception x)
         {
            x.printStackTrace();
         }



         if (event.hasAttribute( MSG))
         {
            msg = event.getString( MSG);

            // An attribute value containing spaces must be single quoted
            // to avoid parsing errors by the TEC event engine.
            if (msg.endsWith( "'"))
            {
               // Remove final quote if already quoted
               msg = msg.substring( 0, msg.length() - 2);
            }
            else
            {
               // Add initial quote if not already quoted
               msg = "'" + msg;
            }

            msg += " [ " + response;


            // Add final message text plus closing quote
            msg += "]'";

         }
         else
         {
            msg = "'[ " + response + " ]'";
         }
      }

      // Update the message field
      event.putItem( MSG, msg);

      // Forward the event
      forward(event);
   }


/******************************************************************************
* Parses any parameters which are part of the action definition
*
* <p> All parameters defined in the rule XML for this action are passed
* in the string paramSz.  For this particular action, the parameters are
* expected to be a series of "key=value" entries separated by white space
* characters. Each (key,value) pair is placed in a Properties object and the
* minimal set of expected keys is:</p>
*
* <ul>
* <li>JDBCDriver
* <li>User
* <li>Password
* <li>DBUrl
* </ul>
*
* <p>Once the parameters have been processed, the information is used to make
* a connection to the specified database using the designated JDBC driver, 
* user name and password.  This connection will be used when processing 
* events.</p>
*
* <p> This method is called once for each rule containing this action.  If 
* the action is shared (<code> singleInstance="true"</code>) then this 
* method is only called once.</p>
*
* @param rule the specific instance of the rule containing this action
* @param paramSz the parameters defined for this action in this rule instance
*
* @see DefaultActionHandler
* @see IRule
*
******************************************************************************/
   public Object doParse(IRule rule, String paramSz) throws ParserException 
   {
      StringTokenizer st = new StringTokenizer( paramSz, " \n\t=");

      String key   = null;
      String value = null;
      try
      {
         // Parse the provided parameters and place them into a Properties object
         while (st.hasMoreTokens())
         {
            key   =  st.nextToken();
            value =  st.nextToken();

            parameters_.put( key, value);
         }


         // Get the JDBC values used to connect to the DB
         String driver = (String) parameters_.get( "JDBCDriver");
         String user   = (String) parameters_.get( "User");
         String pwd    = (String) parameters_.get( "Password");
         String dbUrl  = (String) parameters_.get( "DBUrl");

         // Make the DB Connection
         try
         {
            Class.forName( driver).newInstance();
            dbCon_ = DriverManager.getConnection( dbUrl, user, pwd);

            stmt_ = dbCon_.createStatement();
            stmt_.setMaxRows( 1);
         }
         catch (SQLException ex)
         {
            ex.printStackTrace();
         }
         catch (ClassNotFoundException cnf)
         {
            cnf.printStackTrace();
         }
         catch (Exception x)
         {
            x.printStackTrace();
         }

      }
      catch (NoSuchElementException nse)
      {
         System.out.println( "Missing value for key " + key);
      }

      return null;
   }    




/******************************************************************************
* Gets the value of an event attribute
*
* <p> Checks to see if the event contains the requested attribute and returns
* the value as a String.  If the attribute does not exist then null is
* returned.</p>
*
* @param event the event containing the attribute
* @param attributeName the specific event attribute to get
*
* @return a String representation of the attribute value or null if the event
* does not contain the attribute 
*
* @see Event
*
******************************************************************************/
   private String getEventAttributeValue( Event event, String attributeName)
   {
      String value = null;

      if (event.hasAttribute( attributeName))
      {
         value = event.getString( attributeName);
      }

      return value;
   }



/******************************************************************************
* Determines the value of the key attribute for the event
*
* <p>For the purpose of this example, each event type has an attribute which
* is used to form part of the primary key in the database look up.  This primary
* attribute is something that would indicate a change in the desired response
* such as a NODE being UP or DOWN. The value of this key attribute is returned.</p>
*
* @param event the event containing the attribute
*
* @return a String representation of the attribute value or null if the event
* does not contain the attribute or the event type is not recognized
*
* @see Event
*
******************************************************************************/
   private String getPrimaryEventAttributeValue( Event event)
   {
      String value = null;

      if (event != null)
      {
         if (event.type.equals( "TEC_ITS_NODE_STATUS"))
         {
            value = getEventAttributeValue( event, "nodestatus");
            return value;
         }

         if (event.type.equals( "TEC_ITS_SUBNET_CONNECTIVITY"))
         {
            value = getEventAttributeValue( event, "reachability");
            return value;
         }

         if (event.type.equals( "TEC_ITS_NODE_SERVICE_IMPACT"))
         {
            value = getEventAttributeValue( event, "sub_source");
            return value;
         }
      }

      return null;
   }



/******************************************************************************
* Releases any open resources
*
* <p>Attempts to close the JDBC Statement and the JDBC Connection.</p>
*
******************************************************************************/
   protected void finalize()
   {
      try
      {
         if ( stmt_ != null)
         {
            stmt_.close();
            stmt_ = null;
         }

         if (dbCon_ != null)
         {
            dbCon_.close();
            dbCon_ = null;
         }
      }
      catch (Exception x)
      {
         //Can't really do anything at this point as clean up effort has failed
         x.printStackTrace();
      }
   }
}



