// Actions must be part of the com.tivoli.zce.actions.libs package.
package com.tivoli.zce.actions.libs;

// Minimal set of State Correlation imports
import com.tivoli.zce.IRule;
import com.tivoli.zce.ParserException;
import com.tivoli.zce.CorrelatorException;
import com.tivoli.zce.engine.EventList;
import com.tivoli.zce.engine.Event;
import com.tivoli.zce.actions.DefaultActionHandler;

import com.tivoli.zce.actions.TecEventUtil;

import java.util.Enumeration;
import java.util.NoSuchElementException;
import java.util.Properties;
import java.util.StringTokenizer;

/******************************************************************************
* A sample action which creates a new event
*
* <p>This action parses the parameters provided in the rule XML and incorporates
* the information into the message field of the event.</p>
*
* <p>Once the event is updated the event snapshot is updated by a call to
* <code>TecEventUtil.forwardTecEvent()</code> utility method which does all the 
* additional preparation needed for creating a TEC event and forwarding it.</p> 
*
* <p>Since the snapshot is updated this can be the final action in the rule.  If  
* additional changes to the event are to be made in later actions, the event 
* should just be forwarded here and the <code>SendTecEvent</code> should be 
* listed as the final action in the rule.</p>
*
* @see DefaultActionHandler
* @see com.tivoli.zce.actions.TecEventUtil
* @see ITecEventAttributes
*
******************************************************************************/

public class ParseSample extends DefaultActionHandler implements ITecEventAttributes
{
   private Properties _parameters = new Properties();

/******************************************************************************
* Processes a list of events
*
* <p>Takes the provided list of events and processes each event individually by
* passing it to this actions processEvent() method. </p>
*
* <p> The event list represents the collection of events needed to trigger a
* rule by the State Correlation Engine.  This method is typically called when 
* a state based rule is used and the event list contains the events from the 
* event which triggered the state machine for the rule to the last events 
* processed by the state machine.
*
* @param eventList the list of events to process
*
* @see #processEvent
* @see DefaultActionHandler
* @see EventList
*
******************************************************************************/

   public void processEvents(EventList eventList) throws Exception 
   {
      if (eventList != null)
      {
         try
         {
            for (int i = 0; i < eventList.size(); i++)
            {
               processEvent( eventList.eventAt( i));
            }
         }
         catch (Exception x)
         {
            x.printStackTrace();
         }
      }
   }


/******************************************************************************
* Processes a single event
*
* <p>This action modifies the value of the event attribute "msg" by
* adding the parameters passed in from the XML rule file.</p>
*
* <p>The event is then forwarded to any subsequent actions.</p>
*
* <p>Since the event snapshot is updated in this action it is assumed that
* this action is the final action in the rule.</p>
*
* @param event the event to process
*
* @see #processEvents
* @see DefaultActionHandler
* @see Event
* @see com.tivoli.zce.actions.TecEventUtil
*
******************************************************************************/
   public void processEvent(Event event) throws Exception 
   {
      String msg   = "";
      String key   = null;
      String value = null;

      if (event!= null)
      {
         if (event.hasAttribute( MSG))
         {
            msg = event.getString( MSG);
         }

         // An attribute value containing spaces must be single quoted
         // to avoid parsing errors by the TEC event engine.
         if (msg.endsWith( "'"))
         {
            // Remove final quote if already quoted
            msg = msg.substring( 0, msg.length() - 2);
         }
         else
         {
            // Add initial quote if not already quoted
            msg = "'" + msg;
         }

         msg += " [ ";

         if (_parameters != null)
         {
            Enumeration keyList = _parameters.keys();

            while (keyList.hasMoreElements())
            {
               key = (String) keyList.nextElement();
               value = _parameters.getProperty( key);
               msg += "(" + key + "," + value + ") ";
            }
         }

         // Add final message text plus closing quote
         msg += "]'";
      }

      event.putItem( MSG, msg);


      // Update the event snapshot and forward the event
      // Since this is not a new event the current event
      // is passed as both the new and original event
      TecEventUtil.forwardTecEvent( this, event, event);
   }


/******************************************************************************
* Parses any parameters which are part of the action definition
*
* <p> All parameters defined in the rule XML for this action are passed
* in the string paramSz.  For this particular action, the parameters are
* expected to be a series of "key=value" entries separated by white space
* characters. Each (key,value) pair is placed in a Properties object.</p>
*
* <p> This method is called once for each rule containing this action.  If 
* the action is shared (<code> singleInstance="true"</code>) then this 
* method is only called once.</p>
*
* @param rule the specific instance of the rule containing this action
* @param paramSz the parameters defined for this action in this rule instance
*
* @see DefaultActionHandler
* @see IRule
*
******************************************************************************/
   public Object doParse(IRule rule, String paramSz) throws ParserException 
   {
      StringTokenizer st = new StringTokenizer( paramSz, " \n\t=");

      String key   = null;
      String value = null;
      try
      {
         while (st.hasMoreTokens())
         {
            key   =  st.nextToken();
            value =  st.nextToken();

            _parameters.put( key, value);
         }
      }
      catch (NoSuchElementException x)
      {
         System.out.println( "Missing value for key " + key);
      }

      return null;
   }    

}



